
.packageName <- 'EMMIX'

.First.lib<-function(lib,pkg) { library.dynam('EMMIX',pkg,lib) }


emmix<-function(dat, g, distr = "mvn",ncov    = 3,   
clust   = NULL,init    = NULL,  
itmax   = 1000,epsilon = 1e-6,nkmeans = 5, nrandom = 5, nhclust = FALSE ,
debug = TRUE)
{

if(!is.null(init)  | !missing(init))
obj<-emmixfit2(dat,g, init, distr,ncov,itmax,epsilon)
else
{
if(is.null(clust) | missing(clust)) {

init<- try(init.mix(dat,g,distr,ncov,nkmeans,nrandom,nhclust))

obj<-emmixfit2(dat,g, init, distr,ncov,itmax,epsilon)
}
else {
obj<-emmixfit1(dat,g,clust, distr,ncov,itmax,epsilon)
}
}

error <- obj$error;

ret<-NULL

msg <- switch(error,'1' = paste("stopped at (did not converge within) ", itmax, " iterations"),
'2' = paste("failed to find initial values! "),'3' = paste("estep fails (singularity)"))

if(error & debug)
{
cat('\n-----------------------\n\n')
cat(msg,"\n")
cat('\n-----------------------\n\n')
}


if(error<=1) {


lk<-obj$lk

ICL <- getICL(dat,nrow(dat),ncol(dat),g,distr,ncov,obj$pro,obj$mu,obj$sigma,obj$dof,obj$delta,obj$clust)

ret<-obj

ret$ICL<-ICL$ICL

if(debug) {

msg<-switch(distr, 
'mvn'=paste(g,"- Component Multivariate Normal Mixture Model"),
'mvt'=paste(g,"- Component Multivariate t      Mixture Model"),
'msn'=paste(g,"- Component Multivariate Skew Normal Mixture Model"),
'mst'=paste(g,"- Component Multivariate Skew t      Mixture Model"))


cat('\n-----------------------\n\n')
cat(msg,"\n")
cat('\n-----------------------\n\n')
 
switch(distr, 'mvn' = print(obj[1:7]),'mvt' = print(obj[1:8]),
'msn' = print(obj[c(1:7,9)]),'mst' = print(obj[1:9]))
print(ICL)
cat('\n-----------------------\n')
}

}


ret
}

getICL<-function(x,n,p,g,distr,ncov,pro,mu,sigma,dof,delta,clust)
{

x<-as.matrix(x);loglik<-0;nc=0


lnden<-(as.matrix((ddmix(x, n,p,g, distr, mu, sigma, dof, delta))))

for(h in 1:g) loglik=loglik+sum(ifelse(clust==h,log(pro[h])+lnden[,h],0) )

nc<-switch(ncov,
    '1' = (g-1)+g*p+p*(1+p)/2,  #common covariance
    '2' = (g-1)+g*p+p,          #common diagonal covariance
    '3' = (g-1)+g*(p+p*(1+p)/2),#general covariance
    '4' = (g-1)+g*(p+p),        #general diagonal covariance
    '5' = (g-1)+g*(p+1)  )      #sigma(h)*I_p

nc <- switch(distr,"mvn" = nc,"mvt" = nc + g,"msn" = nc + g*p,"mst" = nc + g*p + g)

ICL = loglik - nc/2*log(n)

list(ICL=ICL)
}



emmixDA<-function(test, g, distr, ncov, 
training=list(data=NULL,cluster=NULL), popPAR = NULL,itmax=500,epsilon=0.0001)
{

if(missing(training)) 
{
  if(missing(popPAR)) stop("either 'training' or 'popPAR' shoud be given")
}
else
  popPAR<-emmixda(training$data,g,distr,ncov,training$cluster,itmax,epsilon)

emmixpred(test,g,distr,popPAR$pro,popPAR$mu,popPAR$sigma,popPAR$dof,popPAR$delta)
}


emmixda<-function(dat,g,distr,ncov,clust,itmax,epsilon)
{
ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4 | ncov<0 | ncov >5) 
stop("model specified not available yet")

dat <- as.matrix(dat)
n   <- nrow(dat)
m   <- ncol(dat)

tau <- e1 <-e2<-e3<-e4<- array(0,c(g*n))

obj<-.Fortran('emskewda',PACKAGE="EMMIX",
as.double(dat),as.integer(n),as.integer(m),
as.integer(g),as.integer(ncov),as.integer(ndist),
pro  = double(g),mu  = double(g*m),sigm  = double(g*m*m),
dof   = double(g),delta = double(g*m),
as.double(tau),as.double(e1),as.double(e2),as.double(e3),as.double(e4),
loglik= double(1),lk=double(itmax),as.integer(clust),
as.integer(itmax),as.double(epsilon),error = integer(1))[c(7:11,17:18,22)]

if(obj$error !=0) stop(paste("error:",obj$error))

lk<-obj$lk;lk<-lk[lk!=0]

list(error=obj$error,
pro=obj$pro,mu= array(obj$mu,c(m,g)),
sigma=array(obj$sigm,c(m,m,g)),dof=obj$dof,
delta=array(obj$delta,c(m,g)),loglik=obj$loglik,lk=lk)
}

emmixpred<-function(dat,g,distr,pro,mu,sigma,dof=NULL,delta=NULL)
{
ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4) 
stop("model specified not available yet")

dat<-as.matrix(dat)
n<-nrow(dat)
p<-ncol(dat)


if(is.null(dof) | missing(dof))
dof <- rep(4,g)
if(is.null(delta) | missing(delta) )
delta <- array(0,c(p,g))

#is mu,sigma,dof,delta specified correctly?

if(length(c(mu)) != (p*g))
stop(paste("mu should be a ",p, 'by', g, "matrix!"))

if(length(c(sigma)) != (p*p*g))
stop(paste("sigma should be a ",p, 'by', p,'by', g, " array!"))

if(length(c(dof)) != g)
stop(paste("dof should be a ",g, " vector!"))

if(length(c(delta)) != (p*g))
stop(paste("delta should be a ",p, 'by', g, " array!"))

tau <-  array(0,c(g*n))


obj<-.Fortran('emskewpred',PACKAGE="EMMIX",
as.double(dat),as.integer(n),as.integer(p),as.integer(g),as.integer(ndist),
as.double(pro),as.double(mu),as.double(sigma),as.double(dof),
as.double(delta),tau= as.double(tau),cluster = integer(n), error= integer(1))[11:13] 


if(obj$error !=0) stop(paste("error:",obj$error))

list(tau = array(obj$tau, c(n,g)),belonging = obj$cluster)
}

emmixMOD<-function(p,g,distr,step,mu,sigma,dof,delta)
{

ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)
if(ndist > 4) 
stop("model specified not available yet")

if(is.null(dof)| missing(dof))
dof <- rep(4,g)
if(is.null(delta) | missing(delta))
delta <- array(0,c(p,g))

if(ndist==3|ndist==1) dof<-rep(0,g)

#is mu,sigma,dof,delta specified correctly?

if(length(c(mu)) != (p*g))
stop(paste("mu should be a ",p, 'by', g, "matrix!"))

if(length(c(sigma)) != (p*p*g))
stop(paste("sigma should be a ",p, 'by', p,'by', g, " array!"))

if(length(c(dof)) != g)
stop(paste("dof should be a ",g, " vector!"))

if(length(c(delta)) != (p*g))
stop(paste("delta should be a ",p, 'by', g, " array!"))

e1 <- e2 <- array(0,c(g*p))
e3 <- array(0,c(g*p*p))


obj<-.Fortran('emskewmod',PACKAGE="EMMIX",
as.integer(ndist),as.integer(p),as.integer(g),as.double(step),
as.double(mu),as.double(sigma),as.double(dof),as.double(delta),
emu = as.double(e1),esigm = as.double(e3),modpt = as.double(e2),
error = integer(1))[9:12]

if(obj$error !=0) stop(paste("error:",obj$error))

e3<- array(obj$esigm,c(p,p,g))

if(ndist ==2) {
for(h in 1:g){
e3[,,h] <- e3[,,h]*dof[h]/(dof[h]-2)}}

list(emu= array(obj$emu,c(p,g)),
esigma=e3,modpts=array(obj$modpt,c(p,g)))
}

emmixfit <-function(dat,g,clust,init,distr,ncov,itmax,epsilon)
{
if(is.null(init) | missing(init))
emmixfit1(dat,g,clust,distr,ncov,itmax,epsilon)
else
emmixfit2(dat,g, init, distr,ncov,itmax,epsilon)
}


emmixfit1<-function(dat,g,clust,distr,ncov,itmax,epsilon)
{
ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)
if(ndist>4|ncov<1|ncov>5) 
stop("the model specified is not available yet")

dat<-as.matrix(dat)
n <- nrow(dat)
p <- ncol(dat)

if(n <= 20*p*p*g)
stop("sample size is too small")

if(missing(clust) | is.null(clust))
stop("initial clust must be given")

tau <- e1 <-e2<-e3<-e4<- array(0,c(g*n))

obj<-.C('emmixfit1',PACKAGE="EMMIX",
as.double(dat),as.integer(n),as.integer(p),
as.integer(g),as.integer(ncov),as.integer(ndist),
pro   = double(g),mu  = double(g*p),sigm  = double(g*p*p),
mdof   = double(g),mdelta= double(g*p),
tau   = as.double(tau),as.double(e1),as.double(e2),as.double(e3),as.double(e4),
loglik= double(1),lk= double(itmax),aic= double(1),bic= double(1),
clust = as.integer(clust),
error = integer(1),as.integer(itmax),as.double(epsilon),as.integer(1))[c(7:12,17:22)]


lk<-obj$lk;lk<-lk[lk!=0]
list(error=obj$error,loglik=obj$loglik,bic=obj$bic,aic=obj$aic,
pro=obj$pro,mu= array(obj$mu,c(p,g)),
sigma=array(obj$sigm,c(p,p,g)),dof=obj$mdof,delta=array(obj$mdelta,c(p,g)),
clust=obj$clust,tau=array(obj$tau,c(n,g)),lk=lk)
}


emmixfit2<-function(dat,g, init, distr,ncov,itmax,epsilon)
{
ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4|ncov<1|ncov>5) 
stop("the model specified is not available yet")


dat<-as.matrix(dat)
#
n <- nrow(dat)
p <- ncol(dat)

if(n <= 20*p*p*g)
stop("sample size is too small")

tau <- e1 <-e2<-e3<-e4<- array(0,c(g*n))

clust <- 1:n

if(is.null(init))
stop("init should be provided")

pro   <- init$pro
mu    <- init$mu
sigma <- init$sigma
dof   <- init$dof
delta <- init$delta


if(is.null(dof))
dof <- rep(4,g)
if(is.null(delta))
delta <- array(0,c(p,g))

if(is.null(mu) | length(c(mu)) != (p*g))
stop(paste("mu should be a ",p, 'by', g, "matrix!"))

if(is.null(sigma) | length(c(sigma)) != (p*p*g))
stop(paste("sigma should be a ",p, 'by', p,'by', g, " array!"))

if(is.null(dof) | length(c(dof)) != g)
stop(paste("dof should be a ",g, " vector!"))

if(is.null(delta) | length(c(delta)) != (p*g))
stop(paste("delta should be a ",p, 'by', g, " array!"))



obj<-.C('emmixfit2',PACKAGE="EMMIX",
as.double(dat),as.integer(n),as.integer(p),
as.integer(g),as.integer(ncov),as.integer(ndist),
pro   = as.double(pro),mu  = as.double(mu),sigm  = as.double(sigma),
mdof   = as.double(dof),mdelta= as.double(delta),
tau   = as.double(tau),as.double(e1),as.double(e2),as.double(e3),as.double(e4),
loglik= double(1),lk= double(itmax),aic= double(1),bic= double(1),
clust = as.integer(clust),
error = integer(1),as.integer(itmax),as.double(epsilon),as.integer(1))[c(7:12,17:22)]


lk<-obj$lk;lk<-lk[lk!=0]
list(error=obj$error,loglik=obj$loglik,bic=obj$bic,aic=obj$aic,
pro=obj$pro,mu= array(obj$mu,c(p,g)),
sigma=array(obj$sigm,c(p,p,g)),dof=obj$mdof,delta=array(obj$mdelta,c(p,g)),
clust=obj$clust,tau=array(obj$tau,c(n,g)),lk=lk)
}




#*****************************************
#initial values
#*****************************************


initEmmix<-function(dat,g,clust,distr,ncov,maxloop=10)
{
ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4|ncov<1|ncov>5) 
stop("the model specified is not available yet")

dat<-as.matrix(dat)
#
n <- nrow(dat)
p <- ncol(dat)

if(n <= 20*p*p*g)
stop("sample size is too small")

tau <- e1 <-e2<-e3<-e4<- array(0,c(g*n))

obj<-.C('initfit_',PACKAGE="EMMIX",
as.double(dat),as.integer(n),as.integer(p),
as.integer(g),as.integer(ncov),as.integer(ndist),
pro   =double(g),mu  =double(g*p),sigm  = double(g*p*p),mdof =double(g),delta =double(g*p),
tau   = as.double(tau),as.double(e1),as.double(e2),as.double(e3),as.double(e4),
loglik= double(1),as.integer(clust),error = integer(1),as.integer(1),as.integer(maxloop)) [c(7:11,17,19)]


list(error=obj$error,loglik=obj$loglik,
pro=obj$pro,mu= array(obj$mu,c(p,g)),
sigma=array(obj$sigm,c(p,p,g)),
dof=obj$mdof,delta=array(obj$delta,c(p,g)))

}

init.mix<-function(dat,g,distr,ncov,nkmeans,nrandom,nhclust,maxloop=10)
{
found<-list()
found$loglik<--Inf

n<-nrow(dat)
clust<-rep(1,n)
mclust<-NULL


if(g>1){
if(nkmeans>0) {


for(i in 1:nkmeans)
{
clust<-kmeans(dat,g,nstart=5)$cluster

#if(min(table(clust))<10) next

obj<-try(initEmmix(dat,g,clust,distr,ncov,maxloop))

if(length(obj)!=7 | obj$error) next

if(obj$loglik>found$loglik)
{
found<-obj
mclust<-clust
}

}

if(is.null(mclust))
nrandom <- 10
}



if(nrandom>0)
for(i in 1:nrandom) {
clust<-sample(1:g,n,replace=T)

#if(min(table(clust))<10) next

obj<-try(initEmmix(dat,g,clust,distr,ncov,maxloop))
if(length(obj)!=7 | obj$error!=0) next
if(obj$loglik>found$loglik)
{
found<-obj 
mclust<-clust
}

} 

#methods<-c( "ward", "single", "complete", "average", "mcquitty", "median","cen")
methods<-c("complete")
#
if(nhclust)
{
dd <- as.dist((1 - cor(t(dat)))/2)  
#Use correlations between variables ``as distance''

for(j in methods){	
clust<- cutree(hclust(dd, j), k = g)

#if(min(table(clust))<10) next

obj<-try(initEmmix(dat,g,clust,distr,ncov,maxloop))
if(length(obj)!=7 | obj$error!=0) next

if(obj$loglik>found$loglik)
{
found<-obj;
mclust<-clust
}

} #end of for j
} #end if
#------------------------------------------------
}
else
{
obj<-try(initEmmix(dat,g,clust,distr,ncov,maxloop))
if(length(obj)!=7 | obj$error!=0)  
stop("failed to find a initial values")
found<-obj;mclust<-clust
}

if(is.null(mclust)) stop("failed to find a initial values!")

found
}



rdmvn<-function (n, p,mean = rep(0,p), cov = diag(p)) 
{
cov<-as.matrix(cov)
    
if (nrow(cov) != ncol(cov)) {
        stop("cov must be a square matrix")
    }
    if (length(mean) != nrow(cov)) {
        stop("mean and cov have non-conforming size")
    }

rmvnorm(n, mean = mean, sigma = cov,method="chol")

}


rdmvt<-function(n,p,mean = rep(0,p),cov=diag(p),nu=3)
{
cov<-as.matrix(cov)
u<-rgamma(n,nu/2,nu/2)
t(t(rdmvn(n,p,cov=cov)/sqrt(u))+mean)
}


rdmsn<-function(n,p,mean=rep(0,p),cov=diag(p),del=rep(0,p))
{

x<-rdmvn(n,p,mean,cov)
z<-abs(rnorm(n))
as.matrix(z%*%t(del)+x)

}



rdmst<-function(n,p,mean = rep(0,p), cov=diag(p),nu=10,del = rep(0,p))
{
u<-rgamma(n,nu/2,nu/2)
x<-t(t(rdmvn(n,p,cov=cov)/sqrt(u))+mean)
z<-abs(rnorm(n)/sqrt(u))
as.matrix(z%*%t(del)+x)
}



rdemmix2<-function(
n,p,g,distr,pro,mu,sigma,dof=NULL,delta=NULL)
{

nn <- table(sample(1:g, n, replace = TRUE, prob = pro))
names(nn) <- NULL


rdemmix(nn,p,g,distr,mu,sigma,dof,delta)

}

rdemmix3<-function(n,p,g,distr,pro,mu,sigma,dof=NULL,delta=NULL)
{

if(length(pro) != g)
stop(paste("pro should be a ",g, " vector!"))

nn <- table(sample(1:g, n, replace = TRUE, prob = pro))
names(nn) <- NULL

dat <- rdemmix(nn,p,g,distr,mu,sigma,dof,delta)

list(data = dat, cluster = rep(1:g,nn) )

}




rdemmix<-function(
nvect,p,g,distr,mu,sigma,dof=NULL,delta=NULL)
{

if(length(c(nvect))!=g) stop("nvect should be a vector")

ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4) 
stop("the model specified is not available yet")

if(is.null(dof))
dof <- rep(4,g)

if(is.null(delta))
delta <- array(0,c(p,g))

if(length(c(mu)) != (p*g))
stop(paste("mu should be a ",p, 'by', g, "matrix!"))

if(length(c(sigma)) != (p*p*g) | dim(sigma)[1] !=p)
stop(paste("sigma should be a ",p, 'by', p,'by', g, " array!"))

if(length(c(dof)) != g)
stop(paste("dof should be a ",g, " vector!"))

if(length(c(delta)) != (p*g) )
stop(paste("delta should be a ",p, "by", g, " array!"))

#

dat<-array(0,c(10,p))

mvrand<-function(n,p,ndist,mean,cov,nu,del)
{

switch(ndist,
'1' = rdmvn(n,p,mean=mean,cov=cov              ),
'2' = rdmvt(n,p,mean=mean,cov=cov,nu=nu        ),
'3' = rdmsn(n,p,mean=mean,cov=cov,      del=del),
'4' = rdmst(n,p,mean=mean,cov=cov,nu=nu,del=del))
}


if(g>=1)
for(h in 1:g)
{
if(nvect[h]>0)
dat<-rbind(dat,mvrand(nvect[h],p,ndist,mu[,h],sigma[,,h],dof[h],delta[,h]))

}



dat[-(1:10),]
}


ddmvn<-function(dat,n,p,mean = rep(0,p),cov = diag(p)                   )
{
exp(ddmix(dat,n,p,1,"mvn", mean,cov,0,rep(0,p)))

}

ddmvt<-function(dat,n,p,mean = rep(0,p),cov=diag(p),nu=4                )
{
exp(ddmix(dat,n,p,1, "mvt", mean,cov,nu,rep(0,p)))
}

ddmsn<-function(dat,n,p,mean=rep(0,p),cov=diag(p),       del = rep(0,p))
{
exp(ddmix(dat,n,p,1, "msn", mean,cov,0,del))
}

ddmst<-function(dat,n,p,mean = rep(0,p),cov=diag(p),nu=4,del = rep(0,p))
{
exp(ddmix(dat,n,p,1, "mst", mean,cov,nu,del))
}



ddmix <- function(dat,n,p,g,distr, mu, sigma, dof=NULL, delta=NULL)
{

if(is.null(dof))
dof <- rep(4,g)

if(is.null(delta))
delta <- array(0,c(p,g))


ndist<-switch(distr,"mvn"=1,"mvt"=2,"msn"=3,"mst"=4,5)

if(ndist>4) 
stop("the model specified is not available yet")

dat<-as.matrix(dat);

if(n == 1 & (ncol(dat) ==1))
dat<-t(dat)

if(nrow(dat)!=n | ncol(dat)!=p )
stop("dat does not match n and p.")

#is mu,sigma,dof,delta specified correctly?

if(length(c(mu)) != (p*g))
stop(paste("mu should be a ",p, 'by', g, "matrix!"))

if(length(c(sigma)) != (p*p*g))
stop(paste("sigma should be a ",p, 'by', p,'by', g, " array!"))

if(length(c(dof)) != g)
stop(paste("dof should be a ",g, " vector!"))

if(length(c(delta)) != (p*g))
stop(paste("delta should be a ",p, 'by', g, " array!"))

den <- array(0,c(n,g))


obj<-.C('ddmix',PACKAGE="EMMIX",
as.double(dat),as.integer(n),
as.integer(p),as.integer(g),as.integer(ndist),
as.double(mu),as.double(sigma),
as.double(dof),as.double(delta),
den = as.double(den),error = integer(1))[10:11] 

if(obj$error) stop("error")

(matrix(obj$den,ncol=g))

}




error.rate<-function(clust1,clust2)
{
if((n=length(clust1))!=length(clust2))
stop("error: length not equal")
if( (g=length(table(clust1)))!=length(table(clust2)))
stop("the number of clusters are not equal")


permute<-function(a)
{
n<-length(a)
if(n==1)
f<-a
else
{
nm<-gamma(n)
f<-array(0,c(n,n*nm))
j<-1

for(i in a)
{
 f[1, (j-1)*nm+1:nm]<-i
 f[-1,(j-1)*nm+1:nm]<-permute(setdiff(a,i))
 j<-j+1
}
}

f
}


#
id<-1:n
cmb<-permute(1:g)
nperm<-ncol(cmb)
rate<-rep(0,nperm)
#
for(i in 1:nperm)
{
tmp<-rep(0,g)
tc<-rep(0,n)
for(j in 1:g)
tc[clust2==j]=cmb[j,i]

for(j in 1:g)
{  
tmp1<-0 
for(k in (1:g)[-j])
        tmp1<-tmp1+length(intersect(id[clust1==j],id[tc==k]))
tmp[j]<-tmp1
}
rate[i]<-sum(tmp)/n
}
min(rate)
}


# BOOTSTRAP functions

bootstrap <- function(x,n,p,g,distr,ncov,popPAR,B=99, replace=TRUE,itmax=1000,epsilon=1e-5)
{
if(missing(popPAR))
stop("please run the function EMMIX() first")

counter <- 0
nnn <- g*(1 + p + p*p + 1 + p) 

ret <- array(0, c(B,nnn)) 

dimnames(ret) <- list(1:B, c(
paste("pi",1:g,sep=''),
paste("mu",rep(1:p,g),rep(paste(1:g,sep=''),rep(p,g)),sep=''),
paste("sigma",rep(paste(rep(1:p,rep(p,p)),rep(1:p,p),sep=''),g),rep(paste(',',1:g,sep=''),rep(p*p,g)),sep=''),
paste("dof",1:g,sep=''),
paste("delta",rep(1:p,g),rep(paste(1:g,sep=''),rep(p,g)),sep='')))



for(i in 1:(2*B) )
{

if(replace)
dat <- x[sample(1:n,n,replace=TRUE),]
else
dat <- rdemmix3(n,p,g,distr,popPAR$pro,popPAR$mu,popPAR$sigma,popPAR$dof,popPAR$delta)


obj <- emmixfit2(dat,g, popPAR, distr,ncov,itmax,epsilon)

if(obj$error > 1) next

counter <- counter +1 

ret[counter,] <- c(obj$pro,obj$mu,obj$sigma,obj$dof,obj$delta)

if(counter >= B) break 

}

std <- sqrt(apply(ret[1:counter,],MARGIN=2,FUN= "var"))

names(std) <- dimnames(ret)[[2]]
std
}

bootstrap.noc <- function(x,n,p,g1,g2,distr,ncov,B=99, replace=TRUE,itmax=1000,epsilon=1e-5)
{
if(g1 >= g2)
stop("g1 should be less than g2")


counter <- 0

vlk <- rep(0,g2-g1+1)

ret <- array(0,c(B,g2-g1))

dimnames(ret) <- list(1:B,paste(g1:(g2-1),"vs",(1+g1:(g2-1)),sep=' '))

lk0 <- -Inf
# start

for( g in g1:g2) {

counter <- 0

lk1 <- -Inf

while(counter < 10) {

clust <- kmeans(x,g,nstart=5)$cluster

emobj <- emmixfit1(x,g,clust, distr,ncov,itmax,epsilon)

if(emobj$error>1) next

if(emobj$loglik > lk1) 
lk1 <- emobj$loglik 

counter = counter +1

}

counter <- 0

lk0 <- lk1

vlk[g-g1+1] <- lk0

if(g < g2) {


for(i in 1:(2*B) )
{

if(replace)
dat <- x[sample(1:n,n,replace=TRUE),]
else
dat <- rdemmix2(n,p,g,distr,emobj$pro,emobj$mu,emobj$sigma,emobj$dof,emobj$delta)

if(is.null(dat)) stop("I can not generate the data!")

obj <- emmixfit2(dat,g, emobj, distr,ncov,itmax,epsilon)

if(obj$error > 1) next

ii <- 0

lk2 <- -Inf

while(ii<10) {

clust <- kmeans(dat,g+1,nstart=5)$cluster

obj2 <- emmixfit1(dat,g+1,clust, distr,ncov,itmax,epsilon)

ii <- ii + 1

if(obj2$error>1) next

if(obj2$loglik > lk2)
lk2 <- obj2$loglik 

} #end ii loop


counter <- counter +1 

ret[counter,g-g1+1] <- -2*(obj$loglik-lk2)

if(counter >= B) break 

} #end i loop
} # end g loop

}# end if

pvalue <- rep(0,g2-g1)

for(i in 1:(g2-g1))
{
pvalue[i] <- sum(ret[,i] < 2*(vlk[i+1]-vlk[i]))/B
}

list(ret=ret,vlk=vlk,pvalue=pvalue)
}



# distance functions


# calculate the Scale free Weighted (mahalonobis distance) Ratio (SWR)and UWR
getSWR<-function(dat,g,sigma,clust, tau)
{

ret <- NULL

if(g>1)
{

intra <- intradist(dat,g,sigma, clust, tau) 
if(intra$error) stop("error")

inter <- interdist(dat,g,sigma, clust, tau) 
if(inter$error) stop("error")

ret <- list(SWR=sqrt(intra$OverallIntraDist1/inter$OverallInterDist1),
UWR=sqrt(intra$OverallIntraDist2/inter$OverallInterDist2))
}

ret

}

# mahalonobis distance

mahalonobis<-function(p, g, mu, sigma) 
{


obj<-.C('mahalonobis_',PACKAGE="EMMIX",
as.integer(p),as.integer(g),as.double(mu),as.double(sigma), 
distance = double(g*g), error = integer(1)) 

if(obj$error) stop("") 

matrix(obj$distance, ncol=g)


}


intradist<-function(dat,g,sigma, clust, tau) 
{
dat<-as.matrix(dat)

intraobj<-.C('intradist_',PACKAGE="EMMIX",
as.double(dat),as.integer((n=nrow(dat))),as.integer((m=ncol(dat))),
as.integer(g),as.integer(clust),as.double(sigma),as.double(tau),
dist1=double(g+1),dist2 = double(g+1), error = integer(1)) 

list(error=intraobj$error,dist1 = intraobj$dist1[1:g],dist2 = intraobj$dist2[1:g],
OverallIntraDist1=intraobj$dist1[g+1],OverallIntraDist2=intraobj$dist2[g+1])
}


interdist<-function(dat,g,sigma, clust, tau) 
{

dat<-as.matrix(dat)


interobj<-.C('interdist_',PACKAGE="EMMIX",
as.double(dat),as.integer((n=nrow(dat))),as.integer((m=ncol(dat))),
as.integer(g),as.integer(clust),as.double(sigma),as.double(tau),
dist1=double(g*g+1),dist2 = double(g*g+1), error = integer(1)) 



list(error=interobj$error,dist1 = matrix(interobj$dist1[1:(g*g)],ncol=g),
dist2 = matrix(interobj$dist2[1:(g*g)],ncol=g), 
OverallInterDist1 = interobj$dist1[(g*g)+1],
OverallInterDist2 = interobj$dist2[(g*g)+1]   )

}



# the end


